Ext.namespace('dSS11', 'dSS11.config');

function _(msgid)
{
    if (dSS11.config.gt === undefined) {
        return msgid;
    }

    return dSS11.config.gt.gettext(msgid);
}

Ext.namespace('dSS11', 'dSS11.config');

dSS11.NetConfigPanel = Ext.extend(Ext.Panel,
    {
        initComponent: function()
        {
            this.url = '/cgi-bin/netinfo.sh';

            this.backedUp = false;

            this.formData = {};

            this.ipset = new Ext.form.FieldSet(
                {
                    layout: 'form',
                    ref:    'ipset',
                    scope: this,
                    items: [
                            new Ext.form.TextField({
                                 width: 140,
                                 name: 'ip',
                                 allowBlank: false,
                                 fieldLabel: _("IP Address"),
                                 validator: this.validateIP
                            }),
                            new Ext.form.TextField({
                                 width: 140,
                                 name: 'netmask',
                                 allowBlank: false,
                                 fieldLabel: _("Netmask"),
                                 validator: this.validateIP
                            }),
                            new Ext.form.TextField({
                                 width: 140,
                                 name: 'gateway',
                                 allowBlank: true,
                                 fieldLabel: _("Gateway"),
                                 validator: this.validateIPBlankOK.createDelegate(this)
                            }),
                            new Ext.form.TextField({
                                 width: 140,
                                 name: 'dns1',
                                 allowBlank: true,
                                 fieldLabel: _("DNS") + "1",
                                 validator: this.validateIPBlankOK.createDelegate(this)
                            }),
                            new Ext.form.TextField({
                                 width: 140,
                                 name: 'dns2',
                                 allowBlank: true,
                                 fieldLabel: _("DNS") + "2",
                                 validator: this.validateIPBlankOK.createDelegate(this)
                            }),
                            new Ext.form.DisplayField({
                                width: 140,
                                ref: 'mac',
                                name: 'mac',
                                fieldLabel: _("MAC")
                            })
                        ] // ipset items

                }
            );

            this.ipform = new Ext.FormPanel({
                        padding: 6,
                        ref: 'ipform',
                        id: 'ipform',
                        url: '/cgi-bin/netcfgwrapper.sh',
                        items: [
                        new Ext.Panel({
                            autoEl: 'span',
                            cls: 'x-green-text',
                            autoLoad: dSS11.config.langPath + 'netconf.text',
                            bodyBorder: false
                        }),
                            {
                                 xtype: 'panel',
                                 height: 10,
                                 bodyBorder: false
                            },

                            new Ext.CycleButton({
                                width: 400,
                                showText: true,
                                id: 'mode',
                                ref: 'mode',
                                items: [
                                            {
                                                xtype: 'menucheckitem',
                                                text:
                                                _("Automatic IP address " +
                                                    "configuration " +
                                                    "using DHCP"),
                                                id: 'ipdhcp',
                                                ref: 'ipdhcp'
                                            },
                                            {
                                                xtype: 'menucheckitem',
                                                text: _("Static IP address " +
                                                        "configuration"),
                                                id: 'ipstatic',
                                                ref: 'ipstatic'
                                            }
                                ],
                                changeHandler: this.netCycle,
                                scope: this
                            }),
                            {
                                 xtype: 'panel',
                                 height: 10,
                                 bodyBorder: false
                            },
                            {
                                xtype: 'panel',
                                ref: 'message',
                                id: 'message',
                                height: 20,
                                cls: 'x-green-text',
                                autoEl: 'span',
                                bodyBorder: false
                            },
                            this.ipset,
                            {
                                xtype: 'panel',
                                layout: 'border',
                                bodyBorder: false,
                                height: 35,
                                items: [
                                    {
                                        xtype: 'panel',
                                        region: 'center',
                                        bodyBorder: false,
                                        width: 150,
                                        buttonAlign: 'left',
                                        minButtonWidth: 80,
                                        buttons: [
                                            {
                                                xtype: 'button',
                                                text:  _("Refresh"),
                                                handler: this.reloadData.createDelegate(this)
                                            }
                                        ]
                                    },
                                    {
                                        xtype: 'panel',
                                        region: 'east',
                                        bodyBorder: false,
                                        buttonAlign: 'right',
                                        minButtonWidth: 80,
                                        buttons: [
                                        {
                                            xtype: 'button',
                                            text:  _("Apply"),
                                            handler: this.applyChanges.createDelegate(this)
                                        },
                                        {
                                            xtype: 'button',
                                            text:  _("Cancel"),
                                            handler: this.cancelChanges,
                                            scope: this
                                        },
                                        {
                                            xtype: 'button',
                                            text:  _("Help"),
                                            handler: this.help
                                        }
                                    ]
                                }
                              ]
                            }
                        ] // form items
                    });

            this.msgready = false;

            Ext.apply(this, {
                hideBorders: true,
                items: [
                    this.ipform
                ]
            });

            dSS11.NetConfigPanel.superclass.initComponent.apply(this, arguments);
            this.on('activate',
                    function() {
                        this.msgready = true;
                        this.reloadData(1000, this);
                    },
                    this);

            this.on('beforeshow',
                    function() {
                        if (this.backedUp === false) {
                            Ext.Ajax.request({
                                url: '/cgi-bin/netcfgwrapper.sh',
                                disableCaching: true,
                                method: 'GET',
                                scope: this,
                                params: { mode: 'backup' },
                                success: function() {
                                    this.backedUp = true;
                                },
                                failure: function() {}
                            });
                        }
                    },
                    this);

        }, // initcomponent

    reloadData: function() {
        this.el.mask(_("Loading current network configuration") + "...",
                      'x-mask-loading');
        this.ipform.load({
                url: this.url,
                method: "GET",
                scope: this,
                disableCaching: true,
                success: function(form, action) {
                    this.el.unmask();
                    try {
                        var jsonData = Ext.util.JSON.decode(
                                                action.response.responseText);

                        if (jsonData.success !== true) {
                            Ext.Msg.alert(_("Error"), jsonData.message);
                            return;
                        }

                        this.formData = jsonData.data;

                        if (jsonData.data.mode === 'static') {
                            this.ipform.mode.setActiveItem(
                                              this.ipform.mode.menu.ipstatic);
                        } else {
                            this.ipform.mode.setActiveItem(
                                              this.ipform.mode.menu.ipdhcp);
                        }
                    } catch (error) {
                        Ext.Msg.alert(_("Error"),
                            _("Could not load network settings") +
                                ": "  + error.message);
                    }
                },
                failure: function(form, action) {
                    this.el.unmask();
                    switch (action.failureType) {
                        case Ext.form.Action.CONNECT_FAILURE:
                            Ext.Msg.alert(_("Error"),
                                _("Ajax communication failed"));
                        break;
                        default:
                            Ext.Msg.alert(_("Error"),
                                    _("Could not load network settings"));
                        break;
                    }
                }
        });
    },

    validateIP: function(ip) {
        if (ip === undefined) {
            return false;
        }

        ip = ip.replace(/\s/g, "");
        var re = /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/;
        if (re.test(ip)) {
            var parts = ip.split(".");
            if (parts.lengh < 4) {
                return false;
            }

            var n1 = parseInt(parseFloat(parts[0]), 10);
            var n2 = parseInt(parseFloat(parts[1]), 10);
            var n3 = parseInt(parseFloat(parts[2]), 10);
            var n4 = parseInt(parseFloat(parts[3]), 10);

            if (n1 === n2 === n3 === n4 === 255) {
                return false;
            }

            if (n1 === n2 === n3 === n4 === 0) {
                return false;
            }

            if (n1 === 0) {
                return false;
            }

            if (n4 === 255) {
                return false;
            }

            for (var i = 0; i < parts.length; i++) {
                var num = parseInt(parseFloat(parts[i]), 10);
                if ((num > 255) || (num < 0)) {
                    return false;
                }
            }
            return true;
        } else {
            return false;
        }
    },

    validateIPBlankOK: function(ip) {
        if ((ip === undefined) || (ip.length === 0)) {
            return true;
        }

        return this.validateIP(ip);
    },

    netCycle: function(btn, item) {
        if (item.id === 'ipdhcp') {
            if (this.msgready === true) {
                this.ipform.message.update(_("Your network settings will be configured automatically via DHCP."));
            }
            for (var i = 0; i < this.ipset.items.items.length; i++) {
                if (this.ipset.items.items[i].name !== 'mac') {
                    this.ipset.items.items[i].disable();
                }
            }

        } else if (item.id === 'ipstatic') {
            if (this.msgready === true) {
                this.ipform.message.update(_("Please configure your network settings manually:"));
            }
            for (var j = 0; j < this.ipset.items.items.length; j++) {
                this.ipset.items.items[j].enable();
            }

         }
    },
    cancelChanges: function() {
        this.ipform.getForm().setValues(this.formData);

        if ((this.formData.mode === 'static') && (this.ipform.mode.getActiveItem() != this.ipform.mode.menu.ipstatic)) {
            this.ipform.mode.setActiveItem(this.ipform.mode.menu.ipstatic);
        } else if ((this.formData.mode === 'dhcp') && (this.ipform.mode.getActiveItem() != this.ipform.mode.menu.ipdhcp)) {
            this.ipform.mode.setActiveItem(this.ipform.mode.menu.ipdhcp);
        }
    },
    restoreSettings: function() {
        Ext.Ajax.request({
            url: '/cgi-bin/netcfgwrapper.sh',
            disableCaching: true,
            method: 'GET',
            params: { mode: 'restore' },
            scope: this,
            success: function() { this.reloadData(); },
            failure: function() {
                Ext.Msg.alert(_("Error"), _("Could not restore last working configuration!"));
                this.reloadData();
            }
        });
    },
    applyChanges: function() {
        if (!this.ipform.getForm().isValid()) {
            Ext.Msg.alert(_("Error"), _("Please correct the errors in the red marked fields."));
            return;
        }
        this.el.mask(_("Applying changes") + '...',
                      'x-mask-loading');

        var lastSettings = this.formData;

        this.ipform.getForm().submit({
            url: this.ipform.url, method: 'GET',
            scope: this,
            params: { 'mode' : this.ipform.mode.getActiveItem().id },
            success: function(form, action) {
                this.el.unmask();
                try {
                    var jsonData = Ext.util.JSON.decode(
                                            action.response.responseText);

                    if (jsonData.code === dSS11.config.codes.ERR_REDIRECT) {
                        if (jsonData.ip.length > 0) {
                            this.el.mask(_("Redirecting to new location at") +
                                " http://" + jsonData.ip + "/",
                                                          'x-mask-loading');

                            window.location = 'http://' + jsonData.ip +
                                              '/?select=3';
                        }
                    } else {
                        this.reloadData();
                    }
                } catch (error) {
                        Ext.Msg.alert(_("Error"),
                            _("Could not load network settings") +
                                ": "  + error.message);
                }
            },
            failure: function(form, action) {
                this.el.unmask();
                if (action.failureType === Ext.form.Action.CONNECT_FAILURE) {
                    Ext.Msg.alert(_("Error"), _("Ajax communication failed"));
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(
                                            action.response.responseText);
                    if (jsonData.code == dSS11.config.codes.ERR_NETUP) {
                        Ext.Msg.show({
                            title: _("Error"),
                            msg: _("There was an error applying your " +
                                "new network settings! Do you want to " +
                                "revert to your previous working " +
                                "configuration?"),
                                buttons: Ext.Msg.YESNO,
                                scope: this,
                                fn: function(button, text) {
                                    if (button == 'yes') {
                                        this.formData = lastSettings;
                                        this.cancelChanges();
                                        this.restoreSettings();
                                    } else {
                                        this.reloadData();
                                    }
                                },
                                animEl: 'elId',
                                icon: Ext.MessageBox.QUESTION
                            });
                        return;
                    }

                    Ext.Msg.alert(_("Error"), jsonData.message);
                    return;

                } catch (error) {
                        Ext.Msg.alert(_("Error"),
                            _("Could not load network settings") +
                                ": "  + error.message);
                }
            }
        });
    },
    help: function() {
    }
});

Ext.reg('dss11netconfig', dSS11.NetConfigPanel);
Ext.namespace('dSS11', 'dSS11.time');

dSS11.time.dateprint = function(date, month, year, hour, minute, zone) {
    switch (month) {
    case 0:
        month = _("January");
        break;
    case 1:
        month = _("February");
        break;
    case 2:
        month = _("March");
        break;
    case 3:
        month = _("April");
        break;
    case 4:
        month = _("May");
        break;
    case 5:
        month = _("June");
        break;
    case 6:
        month = _("July");
        break;
    case 7:
        month = _("August");
        break;
    case 8:
        month = _("September");
        break;
    case 9:
        month = _("October");
        break;
    case 10:
        month = _("November");
        break;
    case 11:
        month = _("December");
        break;
    default:
        month = "";
        break;
    }

    if (hour < 10) {
        hour = ('0' + hour);
    }

    if (minute < 10) {
        minute = ('0' + minute);
    }

    return (date + '. ' + month + ' ' + year + ', ' + hour + ':' + minute +
        (((zone !== undefined) && (zone.length > 0)) ? ' ' + zone : ''));
}

dSS11.time.update = function() {
    var t = new Date(Date.now() - dSS11.config.dSS11TimeDiff);
    Ext.get('currentTime').update(_("Current date and time") + ": " +
            dSS11.time.dateprint(t.getDate(), t.getMonth(), t.getFullYear(),
                                 t.getHours(), t.getMinutes(),
                                 dSS11.config.timeZoneShort
    ));
    dSS11.config.timeUpdateTimeout = setTimeout('dSS11.time.update();', 1000);
}

Ext.namespace('dSS11', 'dSS11.config');

dSS11.SysConfigPanel = Ext.extend(Ext.Panel,
    {
        initComponent: function()
        {

            this.sysName = new Ext.form.FieldSet(
                {
                    layout: 'form',
                    title:  _("Installation name"),
                    scope:  this,
                    items:  [
                        new Ext.Panel({
                            autoEl: 'span',
                            cls: 'x-green-text',
                            autoLoad: dSS11.config.langPath + 'nameconf.text',
                            bodyBorder: false,
                            footer: true,
                            footerCfg: { html: '<br/>' }
                        }),
                        {
                            layout: 'column',
                            bodyBorder: false,
                            items: [
                                {
                                    layout: 'form',
                                    bodyBorder: false,
                                    width: 335,
                                    items: [
                                        {
                                            xtype: 'textfield',
                                            width: 220,
                                            name: 'hostname',
                                            allowBlank: false,
                                            fieldLabel: _("Name"),
                                            maxLength: 64,
                                            validator: this.validateHostName
                                        }
                                    ]
                                },
                                {
                                    bodyBorder: false,
                                    items: [
                                        {
                                            xtype: 'button',
                                            text: _("Apply"),
                                            handler: this.setSystemName.createDelegate(this)
                                        }
                                    ]
                                }
                            ] // border panel items
                        } // column panel
                    ] // name field set items
                } // name field set
            );

            this.nameForm = new Ext.FormPanel(
                {
                    padding: 6,
                    ref: 'nameform',
                    id: 'nameform',
                    scope: this,
                    items: [
                        this.sysName
                    ]
                }
            );

            this.zoneStore = new Ext.data.JsonStore({
                url: '/cgi-bin/zonelist.sh',
                root: 'data.zonelist',
                fields: [ 'zone' ]
            });

            this.timePanel = new Ext.Panel(
                {
                    layout: 'table',
                    width: 400,
                    layoutConfig: {
                        columns: 1,
                        tableAttrs: {
                            width: '100%'
                        }
                    },
                    scope: this,
                    bodyBorder: false,
                    items: [
                        {
                            xtype: 'checkbox',
                            name: 'ntp',
                            ctCls: 'x-green-text black-text',
                            hideBorders: true,
                            boxLabel: _("Synchronize date and time over the network"),
                            checked: true,
                            listeners: {
                                check: this.NTPChecked.createDelegate(this)
                            }
                        },
                        {
                            xtype: 'panel',
                            bodyBorder: false,
                            layout: 'fit',
                            ref: 'ntpPanelSwitch',
                            items: [
                                {
                                    xtype: 'panel',
                                    layout: 'form',
                                    scope: this,
                                    bodyBorder: false,
                                    ref: 'ntpPanel',
                                    items: [
                                        {
                                            xtype: 'textfield',
                                            fieldLabel: _("NTP server"),
                                            allowBlank: false,
                                            name: 'ntpserver',
                                            width: 220
                                        }
                                    ]
                                }, // ntp panel
                                {
                                    xtype: 'panel',
                                    ref: 'datePanel',
                                    scope: this,
                                    layout: 'form',
                                    bodyBorder: false,
                                    hidden: true,
                                    items: [
                                        {
                                            xtype: 'datefield',
                                            fieldLabel: _("Date"),
                                            ref: 'date',
                                            name: 'setdate',
                                            format: 'd-m-Y',
                                            width: 220
                                        },
                                        {
                                            xtype: 'timefield',
                                            fieldLabel: _("Time"),
                                            ref: 'time',
                                            format: 'H:i:s',
                                            name: 'settime',
                                            width: 220
                                        }
                                    ]
                                } // date panel
                            ]
                        } // panel with all ntp and date/time settings
                    ] // fieldset items
                }
            );

            this.timeSet = new Ext.form.FieldSet(
                {
                    title: _("Date and time settings"),
                    scope: this,
                    layout: 'table',
                    layoutConfig: {
                        columns: 1,
                        tableAttrs: {
                            width: '100%'
                        }
                    },

                    items: [
                        {
                            xtype: 'panel',
                            ref: 'current',
                            id: 'currentTime',
                            cls: 'x-green-text',
                            autoEl: 'span',
                            height: 20,
                            bodyBorder: false
                        },
                        {
                            xtype: 'panel',
                            height: 10,
                            bodyBorder: false
                        },
                        {
                            xtype: 'panel',
                            scope: this,
                            layout: 'form',
                            bodyBorder: false,
                            items: [
                                {
                                    xtype: 'combo',
                                    fieldLabel: _("Time zone"),
                                    lazyInit: false,
                                    mode: 'local',
                                    name: 'timezone',
                                    forceSelection: true,
                                    store: this.zoneStore,
                                    emptyText: _("Select a time zone")+
                                              "...",
                                    triggerAction: 'all',
                                    typeAhead: true,
                                    displayField: 'zone',
                                    valueField: 'zone',
                                    width: 220
                                }
                            ]
                        }, // time zone panel

                        this.timePanel,
                        {
                            bodyBorder: false,
                            xtype: 'panel',
                            width: 335,
                            buttons: [
                                {
                                    xtype: 'button',
                                    text: _("Apply"),
                                    handler: this.applyTimeSettings.createDelegate(this)
                                }
                            ]
                        }
                    ] // timeset items
                }
            );
            this.timeForm = new Ext.FormPanel(
                {
                    padding: 6,
                    ref: 'timeForm',
                    id: 'timeform',
                    scope: this,
                    items: [
                        this.timeSet
                    ]
                }
            );


            this.restart = new Ext.form.FieldSet(
                {
                    title:  _("Restart digitalSTROM server application"),
                    scope:  this,
                    items: [
                        {
                            xtype:  'panel',
                            padding: 3,
                            hideBorders: true,
                            ref: 'dssUptime',
                            bodyBorder: false,
                            cls: 'x-green-text',
                            scope: this,
                            autoLoad: {
                                scope: this,
                                url: '/cgi-bin/dssuptime.hs'
                            }
                        },
                        {
                            xtype: 'button',
                            text: _("Restart dSS"),
                            minWidth: 80,
                            handler: this.restartdSS.createDelegate(this)
                        }
                    ]
                }
            );

            this.reboot = new Ext.form.FieldSet(
                {
                    title:  _("Reboot system"),
                    scope:  this,
                    items: [
                        {
                            xtype:  'panel',
                            padding: 3,
                            bodyBorder: false,
                            hideBorders: true,
                            ref: 'Uptime',
                            cls: 'x-green-text',
                            autoLoad: '/cgi-bin/dss11uptime.hs'
                        },
                        {
                            xtype: 'button',
                            text: _("Reboot"),
                            minWidth: 80,
                            handler: this.reboot.createDelegate(this)
                        }
                    ]
                }
            );

            this.zoneStore.load();

            Ext.apply(this, {
                hideBorders: true,
                items: [
                    this.nameForm,
                    this.timeForm,
                    {
                        layout: 'column',
                        bodyBorder: false,
                        padding: 3,
                        items: [
                            {
                                columnWidth: .5,
                                padding: 3,
                                xtype: 'panel',
                                bodyBorder: false,
                                items: [ this.restart ]
                            },
                            {
                                columnWidth: .5,
                                padding: 3,
                                xtype: 'panel',
                                bodyBorder: false,
                                items: [ this.reboot ]
                            }
                        ]
                    }
                ]
            });

            dSS11.SysConfigPanel.superclass.initComponent.apply(this, arguments);

           this.on('activate',
                    function() {
                        this.reloadData.defer(1000, this);
                    },
                    this);

        }, // initcomponent
        restartdSS: function() {
            Ext.Msg.show({
                title: _("Restart dSS application"),
                msg: _("Are you sure that you want to restart the dSS application?"),
                    buttons: Ext.Msg.YESNO,
                    scope: this,
                    fn: function(button, text) {
                        if (button == 'yes') {
                            this.el.mask(_("Restarting dSS application") +
                                            "...", 'x-mask-loading');

                            Ext.Ajax.request({
                                url: '/cgi-bin/restartdss.sh',
                                disableCaching: true,
                                method: 'GET',
                                scope: this,
                                success: function(response) {
                                    this.el.unmask();
                                    try {
                                        var jsonData = Ext.util.JSON.decode(
                                                response.responseText);

                                        if (jsonData.success !== true) {
                                            Ext.Msg.alert(_("Error"),
                                                            jsonData.message);
                                            return;
                                        } else {
                                            Ext.Msg.show({
                                                title: _("Info"),
                                                msg: _("digitalSTROM " +
                                                "server application " +
                                                "successfully restarted"),
                                                icon: Ext.MessageBox.INFO,
                                                buttons: Ext.Msg.OK
                                            });
                                        }
                                    } catch (error) {
                                        Ext.Msg.alert(_("Error"),
                                            _("digitalSTROM server " +
                                              "application could not be " +
                                              "restarted") + ": " +
                                                error.message);
                                    }
                                    this.restart.dssUptime.doAutoLoad();
                                },
                                failure: function() {
                                    this.el.unmask();
                                    Ext.Msg.alert(_("Error"),
                                        _("digitalSTROM server " +
                                          "application could not be " +
                                          "restarted"));
                                    this.restart.dssUptime.doAutoLoad();
                                }
                            });

                        }
                    },
                    animEl: 'elId',
                    icon: Ext.MessageBox.QUESTION
            });

        },
        rebootWaitMsg: function(secs, w) {
            if (secs > 0) {
                w.updateProgress(1-(secs/dSS11.config.rebootWait), secs + " " +
                        dSS11.config.gt.ngettext("second remaining",
                                                 "seconds remaining",
                                                  secs));
                secs--;
                this.rebootWaitMsg.defer(1000, this, [secs, w]);
            } else {
                w.hide();
                this.findParentByType(Ext.Viewport).el.mask(_("Reloading page") + "...");
                var params = Ext.urlDecode(window.location.search.substring(1));
                params.select = 3;
                window.location = window.location.protocol + '//' +
                                  window.location.host + '/?' +
                                  Ext.urlEncode(params);

            }
        },
        reboot: function() {
            Ext.Msg.show({
                title: _("Reboot system"),
                msg: _("Are you sure that you want to reboot the system?") +
                       "<br/>" +
                       _("Note: this will take about a minute"),
                    buttons: Ext.Msg.YESNO,
                    scope: this,
                    fn: function(button, text) {
                        if (button == 'yes') {
                            Ext.Ajax.request({
                                url: '/cgi-bin/rebootwrapper.sh',
                                disableCaching: true,
                                params: { action : 'reboot' },
                                method: 'GET',
                                scope: this,
                                success: function(response) {
                                    this.el.unmask();
                                    try {
                                        var jsonData = Ext.util.JSON.decode(
                                                response.responseText);

                                        if (jsonData.success !== true) {
                                            Ext.Msg.alert(_("Error"),
                                                            jsonData.message);
                                            return;
                                        } else {
                                            var w = Ext.Msg.progress(
                                                _("Rebooting system"),
                                                _("Please wait while the " +
                                                  "system is rebooting") +
                                                  "<br/>" +
                                                _("The setup page will be " +
                                                  "automatically reloaded " +
                                                  "after the time elapses"));
                                            this.rebootWaitMsg(
                                                    dSS11.config.rebootWait, w);
                                        }
                                    } catch (error) {
                                        Ext.Msg.alert(_("Error"),
                                            _("System reboot could not " +
                                              "be initiated") + ": " +
                                                error.message);
                                    }
                                },
                                failure: function() {
                                    this.el.unmask();
                                    Ext.Msg.alert(_("Error"),
                                        _("System reboot could not be " +
                                          "initiated"));
                                }
                            });

                        }
                    },
                    animEl: 'elId',
                    icon: Ext.MessageBox.QUESTION
            });
        },
        setSystemName:  function() {
            if (!this.nameForm.getForm().isValid()) {
                Ext.Msg.alert(_("Error"), _("The name you entered is invalid! Please do not use any whitespaces or special characters."));
                return;
            }

            this.el.mask(_("Applying changes") + '...', 'x-mask-loading');

            this.nameForm.getForm().submit({
                url: '/cgi-bin/setnamewrapper.sh',
                method: 'GET',
                scope: this,
                success: function(form, action) {
                    this.el.unmask();
                },
                failure: function(form, action) {
                    this.el.unmask();
                    switch (action.failureType) {
                        case Ext.form.Action.CONNECT_FAILURE:
                            Ext.Msg.alert(_("Error"),
                                _("Ajax communication failed"));
                        break;
                        default:
                            Ext.Msg.alert(_("Error"),
                                    _("Could not load installation name"));
                        break;
                    }
                }
            });
        },
        validateHostName: function(name) {
            if ((name.length === 0) || (name.length > 64)) {
                return false;
            }

            var expr = /[^0-9a-zA-Z\-]/;
            if (expr.exec(name) === null) {
                return true;
            }

            return false;
        },
        reloadHostName: function() {
            this.nameForm.el.mask(_("Loading system name") + "...",
                      'x-mask-loading');

            this.nameForm.load({
                url: '/cgi-bin/getname.sh',
                method: 'GET',
                scope: this,
                disableCaching: true,
                success: function(form, action) {
                    try {
                        var jsonData = Ext.util.JSON.decode(
                                                action.response.responseText);

                        if (jsonData.success !== true) {
                            Ext.Msg.alert(_("Error"), jsonData.message);
                            this.nameForm.el.unmask();
                            return;
                        }
                    } catch (error) {
                        Ext.Msg.alert(_("Error"),
                            _("Could not load installation name") +
                                ": "  + error.message);
                    }

                    this.nameForm.el.unmask();
                },
                failure: function(form, action) {
                    switch (action.failureType) {
                        case Ext.form.Action.CONNECT_FAILURE:
                            Ext.Msg.alert(_("Error"),
                                _("Ajax communication failed"));
                        break;
                        default:
                            Ext.Msg.alert(_("Error"),
                                    _("Could not load installation name"));
                        break;
                    }

                    this.nameForm.el.unmask();
                }
            });
        },
        reloadTimeSettings: function() {
            this.timeForm.el.mask(_("Loading date and time settings") + "...",
                      'x-mask-loading');

            this.timeForm.load({
                url: '/cgi-bin/dateinfo.sh',
                method: 'GET',
                scope: this,
                disableCaching: true,
                waitMsgTarget: this.timeForm.el,
                success: function(form, action) {
                    try {
                        var jsonData = Ext.util.JSON.decode(
                                                action.response.responseText);

                        if (jsonData.success !== true) {
                            Ext.Msg.alert(_("Error"), jsonData.message);
                            this.timeForm.el.unmask();
                            return;
                        }

                        dSS11.config.dSS11TimeDiff = dSS11.config.dSS11TimeDiff = (Date.now() - (jsonData.data.utcsecs * 1000));
                        if (dSS11.config.timeUpdateTimeout !== 0) {
                            clearTimeout(dSS11.config.timeUpdateTimeout);
                        }
                        dSS11.config.timeZoneShort = jsonData.data.zoneshort;
                        dSS11.time.update();
                    } catch (error) {
                        Ext.Msg.alert(_("Error"),
                            _("Could not load date and time settings") +
                                ": "  + error.message);
                    }

                    this.timeForm.el.unmask();
                },
                failure: function(form, action) {
                    switch (action.failureType) {
                        case Ext.form.Action.CONNECT_FAILURE:
                            Ext.Msg.alert(_("Error"),
                                _("Ajax communication failed"));
                        break;
                        default:
                            Ext.Msg.alert(_("Error"),
                                    _("Could not load date and time settings"));
                        break;
                    }
                    this.timeForm.el.unmask();
                }
            });
        },
        applyTimeSettings:  function() {
            if (!this.timeForm.getForm().isValid()) {
                Ext.Msg.alert(_("Error"), _("The data you entered is invalid!"));
                return;
            }

            this.el.mask(_("Applying changes") + '...', 'x-mask-loading');

            this.timeForm.getForm().submit({
                url: '/cgi-bin/setdatewrapper.sh',
                method: 'GET',
                scope: this,
                success: function(form, action) {
                    this.el.unmask();
                    this.timePanel.ntpPanelSwitch.datePanel.date.setValue("");
                    this.timePanel.ntpPanelSwitch.datePanel.time.clearValue();
                    this.reloadTimeSettings();
                },
                failure: function(form, action) {
                    this.el.unmask();
                    switch (action.failureType) {
                        case Ext.form.Action.CONNECT_FAILURE:
                            Ext.Msg.alert(_("Error"),
                                _("Ajax communication failed"));
                        break;
                        default:
                            Ext.Msg.alert(_("Error"),
                                    _("Could not set date and time") +
                                    ": " + _(action.result.message));
                        break;
                    }
                }
            });
        },

        reloadData: function() {
            this.reloadHostName();
            this.reloadTimeSettings();
        },
        NTPChecked: function(box, checked) {
            if (checked === true) {
                this.timePanel.ntpPanelSwitch.datePanel.setVisible(false);
                this.timePanel.ntpPanelSwitch.ntpPanel.setVisible(true);
            } else {
                this.timePanel.ntpPanelSwitch.ntpPanel.setVisible(false);
                this.timePanel.ntpPanelSwitch.datePanel.setVisible(true);
            }

        }
    }
);

Ext.reg('dss11sysconfig', dSS11.SysConfigPanel);

Ext.namespace('dSS11', 'dSS11.config');

dSS11.AccessConfigPanel = Ext.extend(Ext.Panel,
    {
        initComponent: function()
        {

            this.webPassword = new Ext.form.FieldSet(
                {
                    layout: 'form',
                    title:  _("Web interface admin password"),
                    scope:  this,
                    items:  [
                        new Ext.Panel({
                            autoEl: 'span',
                            cls: 'x-green-text',
                            autoLoad: dSS11.config.langPath + 'pwinfo.text',
                            bodyBorder: false,
                            footer: true,
                            footerCfg: { html: '<br/>' }
                        }),
                        {
                            xtype: 'textfield',
                            width: 220,
                            name: 'currentpw',
                            ref: 'currentpw',
                            inputType: 'password',
                            allowBlank: false,
                            fieldLabel: _("Current password"),
                            maxLength: 64
                        },
                        {
                            xtype: 'textfield',
                            width: 220,
                            name: 'newpw',
                            ref: 'newpw',
                            inputType: 'password',
                            regex: new RegExp("^\\S+$"),
                            allowBlank: false,
                            fieldLabel: _("New password"),
                            maxLength: 64
                        },
                        {
                            xtype: 'textfield',
                            width: 220,
                            name: 'repeatpw',
                            ref: 'repeatpw',
                            inputType: 'password',
                            submitValue: false,
                            allowBlank: false,
                            regex: new RegExp("^\\S+$"),
                            fieldLabel: _("Repeat new password"),
                            maxLength: 64
                        },
                        {
                            xtype: 'button',
                            text: _("Apply"),
                            handler: this.setPassword.createDelegate(this)
                        }
                    ] // items
                } // field set
            );

            this.passwordForm = new Ext.FormPanel(
                {
                    padding: 6,
                    ref: 'passwordform',
                    id: 'passwordform',
                    scope: this,
                    items: [
                        this.webPassword
                    ]
                }
            );

            Ext.apply(this, {
                hideBorders: true,
                items: [
                    this.passwordForm
                ]
            });

            dSS11.AccessConfigPanel.superclass.initComponent.apply(this, arguments);
        }, // initcomponent
        setPassword:  function() {
            if (!this.passwordForm.getForm().isValid()) {
                Ext.Msg.alert(_("Error"), _("The password is missing or invalid!"));
                return;
            }

            if (this.webPassword.newpw.getValue() !=
                this.webPassword.repeatpw.getValue())
            {
                this.webPassword.newpw.markInvalid();
                this.webPassword.repeatpw.markInvalid();

                Ext.Msg.alert(_("Error"), _("The passwords you entered do not match!"));

                Ext.Msg.show({
                    title:_("Error"),
                    msg: _("The passwords you entered do not match!"),
                    buttons: Ext.Msg.OK,
                    scope: this,
                    fn: function() {
                        this.webPassword.newpw.reset();
                        this.webPassword.repeatpw.reset();
                        this.webPassword.newpw.markInvalid();
                        this.webPassword.repeatpw.markInvalid();
                    }
                });
                return;
            }

            this.el.mask(_("Applying changes") + '...', 'x-mask-loading');

            this.passwordForm.getForm().submit({
                url: '/cgi-bin/setwebpasswrapper.sh',
                method: 'GET',
                scope: this,
                success: function(form, action) {
                    this.webPassword.currentpw.reset();
                    this.webPassword.newpw.reset();
                    this.webPassword.repeatpw.reset();
                    this.el.unmask();
                },
                failure: function(form, action) {
                    this.el.unmask();
                    switch (action.failureType) {
                        case Ext.form.Action.CONNECT_FAILURE:
                            Ext.Msg.alert(_("Error"),
                                _("Ajax communication failed"));
                        break;
                        default:
                            Ext.Msg.alert(_("Error"),
                                    _("Could not set password") + ": " +
                                    _(action.result.message));
                        break;
                    }
                }
            });
        }
    }
);

Ext.reg('dss11accessconfig', dSS11.AccessConfigPanel);

Ext.namespace('dSS11', 'dSS11.config');

dSS11.SystemUpdatePanel = Ext.extend(Ext.Panel,
    {
        initComponent: function()
        {
            this.updateStore = new Ext.data.JsonStore(
                {
                    url: '/cgi-bin/opkgwrapper.sh?action=list-upgradable',
                    root: 'data',
                    fields:
                    [
                        'name', 'version', 'maintainer',
                        'description', 'homepage', 'state'
                    ]
                }
            );

            this.loaded = false;
            this.fromRefetch = false;

            var renderPackage = function(value, p, record)
            {
                var hp = record.get('homepage');
                if (hp !== "") {
                    hp = String.format('&nbsp;&nbsp;<a href="{0}" target="_blank">{0}</a>',
                            hp);
                }

                return String.format('<div class="{0}">' +
                        '<span class="{4}"><b>{1}</b></span>{2}<br/>' +
                        '<span class="{4}">' + _("Version")+
                        ':&nbsp;{3}</span></div>' +
                        '<br/><div class="{4}">{5}' +
                        '</div>',
                        'x-green-text x-text-link',                 // {0}
                        record.get('name'),                         // {1}
                        hp,                                         // {2}
                        record.get('version'),                      // {3}
                        'black-text',                               // {4}
                        record.get('description')                   // {5}
                );
            };

            this.mainGrid = new Ext.grid.GridPanel(
                {
                    store: this.updateStore,
                    columns:
                    [
                        {
                            id: 'addon',
                            header: _("Available updates"),
                            dataIndex: 'name',
                            sortable: true,
                            renderer: renderPackage
                        }
                    ],
                    trackMouseOver: false,
                    viewConfig: { forceFit: true },
                    disableSelection: true,
                    bodyBorder: false
                }
            );

            this.noUpdatesTooltip =  new Ext.ToolTip(
                {
                    target: 'updateButton',
                    html: '<br/>' +
                          _("Your system is up to date"),
                    title: _("No updates found"),
                    closable: true,
                    draggable: true,
                    anchor: 'right'
                }
            );

            this.gotUpdatesTooltip = new Ext.ToolTip(
                {
                    target: 'updateButton',
                    html: '<br/>' +
                          _("Updates are available for your system, click the update button to install them"),
                    title: _("Updates found"),
                    closable: true,
                    draggable: true,
                    anchor: 'right'
                }
            );

            Ext.apply(this,
                {
                    layout: 'fit',
                    tbar: [
                        {
                            xtype: 'button',
                            text: _("Refresh view"),
                            iconCls: 'icon-refresh-view',
                            scope: this,
                            handler: this.refreshList.createDelegate(this)
                        },
                        {
                            xtype: 'button',
                            text: _("Check for new updates"),
                            iconCls: 'icon-refresh-feed',
                            scope: this,
                            tooltip: _("Contact the server and check availability of new updates"),
                            handler: this.handleRefetch.createDelegate(this)
                        },
                        '->',
                        {
                            xtype: 'button',
                            text: _("Update system"),
                            ref: 'updateButton',
                            id: 'updateButton',
                            iconCls: 'icon-package-update',
                            disabled: true,
                            scope: this,
                            handler: this.updateSystem.createDelegate(this)
                        }
                    ],
                    items: [ this.mainGrid ]
                }
            );

            dSS11.SystemUpdatePanel.superclass.initComponent.apply(this, arguments);

            this.on('activate',
                function()
                {
                    if (this.loaded === false) {
                        this.updateStore.load();
                        this.loaded = true;
                    }
                },
                this);

            this.on('hide',
                function()
                {
                    this.noUpdatesTooltip.hide();
                    this.gotUpdatesTooltip.hide();
                },
                this);

            this.updateStore.on('load',
                function()
                {
                    var button = this.getTopToolbar().get('updateButton');
                    var count = this.updateStore.getTotalCount();

                    if (count > 0) {
                        this.noUpdatesTooltip.hide();
                        this.noUpdatesTooltip.disable();
                        this.gotUpdatesTooltip.enable();
                        button.enable();
                        this.gotUpdatesTooltip.setTitle(
                                count + " " +
                                dSS11.config.gt.ngettext("Update",
                                                    "n% Updates", count) +
                                        " " + _("found"));

                        this.gotUpdatesTooltip.show();
                    } else {
                        button.disable();
                        this.gotUpdatesTooltip.hide();
                        this.gotUpdatesTooltip.disable();
                        this.noUpdatesTooltip.enable();
                        if (this.fromRefetch === true) {
                            this.noUpdatesTooltip.show();
                            this.fromRefetch = false;
                        }
                    }
                },
                this);

        },
        refreshList: function()
        {
            this.updateStore.load();
        },
        updateDone: function()
        {
            var msg = _("Your system was successfully updated");
            Ext.Msg.show({
                title: _("Info"),
                msg: msg,
                icon: Ext.MessageBox.INFO,
                buttons: Ext.Msg.OK
            });
            this.refreshList();
        },
        refetchDone: function()
        {
            this.fromRefetch = true;
            this.refreshList();
        },
        handleRefetch: function()
        {
            var fn = dSS11.util.opkg.createDelegate(this,
                           [ 'update',
                              this.findParentByType('viewport').el,
                              this.refetchDone.createDelegate(this) ]);
            fn();
        },
        updateSystem: function() {
            var fn = dSS11.util.opkg.createDelegate(this,
                           [ 'upgrade',
                              this.findParentByType('viewport').el,
                              this.updateDone.createDelegate(this) ]);
            fn();
        }
    }
);

Ext.reg('dss11systemupdate', dSS11.SystemUpdatePanel);

Ext.namespace('dSS11', 'dSS11.config');

dSS11.AddOnsOverviewPanel = Ext.extend(Ext.Panel,
    {
        initComponent: function()
        {
            this.installedStore = new Ext.data.JsonStore(
                {
                    url: '/cgi-bin/opkgwrapper.sh?action=addon-list-installed',
                    root: 'data',
                    fields:
                    [
                        'name', 'version', 'maintainer',
                        'description', 'homepage'
                    ]
                }
            );

            this.loaded = false;

            this.mainView = new Ext.DataView(
                {
                    store: this.installedStore,
                    id: 'add-on-installed-view',
                    scope: this,
                    overClass: 'x-view-over',
                    loadingText: _("Loading list of installed add-ons") + "...",
                    emptyText: _("You do not have any add-ons installed"),
                    deferEmptyText: true,
                    cls: 'x-margined-green-text',
                    tpl: new Ext.XTemplate(
                        '<tpl for=".">',
                        '<div qtip="<b>{name}<br/>Version: ' +
                        '</b>{version}<br/><br/>' +
                        '<i>{description}</i>" style="float: left">',
                        '<div class="addon-cell-item-wrap" id="{name}">',
                        '<div class="addon-cell-item">',
                        '<img src="/add-ons/{name}/default_icon.png"/>',
                        '</div>',
                        '<div class="x-green-text">',
                        '<span>{name}</span>',
                        '</div>',
                        '</div>',
                        '</div>',
                        '</tpl>',
                        '<div class="x-clear"></div>'

                    ),
                    itemSelector: 'div.addon-cell-item-wrap',
                    singleSelect: true,
                    autoScroll: true,
                    listeners: {
                        dblclick: this.forwardToAddon.createDelegate(this)
                    }
                }
            );

            Ext.apply(this,
                {
                    layout: 'fit',
                    tbar: [
                        {
                            xtype: 'button',
                            text: _("Refresh view"),
                            iconCls: 'icon-refresh-view',
                            scope: this,
                            handler: this.refreshOverview.createDelegate(this)
                        },
                        {
                            xtype: 'button',
                            text: _("Open selected"),
                            iconCls: 'icon-package-open',
                            id: 'openButton',
                            tooltip: _("Hint: double click on the add-on icon to open the configuration interface"),
                            handler: this.forwardToSelected.createDelegate(this),
                            disabled: true,
                            scope: this
                        },
                        '->',
                        {
                            text: _("Uninstall selected"),
                            scope: this,
                            tooltip: _("Remove selected add-ons from your system"),
                            iconCls: 'icon-package-delete',
                            handler: this.removeAddons.createDelegate(this),
                            disabled: true,
                            id: 'uninstallButton'
                        }
                    ],
                    items: [ this.mainView ]
                }
            );

            dSS11.AddOnsOverviewPanel.superclass.initComponent.apply(this,
                                                                     arguments);
            this.on('activate',
                function() {
                    if (this.loaded === false) {
                        this.installedStore.load();
                        this.loaded = true;
                    }
                },
                this);

            this.mainView.on('selectionchange',
                function(dv, selections)
                {
                    if (selections.length > 0) {
                        this.getTopToolbar().get('openButton').enable();
                        this.getTopToolbar().get('uninstallButton').enable();
                    } else {
                        this.getTopToolbar().get('openButton').disable();
                        this.getTopToolbar().get('uninstallButton').disable();
                    }
                },
                this);

        }, // initcomponent
        forwardToAddon: function(dv, index, node, ev) {
            window.open('/add-ons/' + dv.getRecord(node).get('name') + '/');
        },
        forwardToSelected: function()
        {
            if (this.mainView.getSelectionCount() === 0) {
                Ext.MessageBox.alert(_("No add-on selected"),
                      _("Please select an add-on to open! You can also double click the add-on icon to open the configuration interface"));
                return;
            }
            var selection = this.mainView.getSelectedRecords();
            var name = selection[0].get('name');
            window.open('/add-ons/' + name + '/');
        },
        removeAddons: function() {
            if (this.mainView.getSelectionCount() === 0) {
                Ext.MessageBox.alert(_("No add-on selected"),
                      _("Please select an add-on that should be uninstalled!"));
                return;
            }
            Ext.Msg.show({
                title: _("Uninstall add-ons"),
                msg: _("Are you sure that you want to uninstall the " +
                       "selected add-on from your system?"),
                buttons: Ext.Msg.YESNO,
                scope: this,
                fn: function(button, text) {
                    if (button === 'yes') {
                        var selection = this.mainView.getSelectedRecords();
                        var name = selection[0].get('name');

                        var fn = dSS11.util.opkg.createDelegate(this,
                            [ 'addon-remove',
                               this.findParentByType('viewport').el,
                               this.doneRemove.createDelegate(this, [ name ]),
                               name ]);
                        fn();
                    }
                }
            });
        },
        refreshOverview: function() {
            this.installedStore.load();
        },
        doneRemove: function(name) {
            Ext.Msg.show({
                title: _("Info"),
                msg: _("Add-on") + " <b>" + name + "</b> " +
                     _("was removed from your system"),
                icon: Ext.MessageBox.INFO,
                buttons: Ext.Msg.OK
            });
            this.refreshOverview();
        }
    }
);

Ext.reg('dss11addonsoverview', dSS11.AddOnsOverviewPanel);
Ext.namespace("dSS11", "dSS11.util");

dSS11.util.checkUpdateProgress = function(updateType, maskElement, doneCallback)
{
    var failMsg = _("Could not get current status");

    if (updateType === 'system') {
        failMsg = _("Could not get update status") + " " +
                  _("It is possible that the system " +
                    "webserver is currently being updated. " +
                    "Please try reloading this page in a " +
                    "few minutes.");
    }

    Ext.Ajax.request({
        url: '/cgi-bin/updatestatuswrapper.sh',
        disableCaching: true,
        method: 'GET',
        scope: this,
        params: { type: updateType },
        success: function(response) {
            try {
                var jsonData = Ext.util.JSON.decode( response.responseText);

                if (jsonData.success !== true) {
                    Ext.Msg.alert(_("Error"), _(jsonData.message));
                    maskElement.unmask();
                    return;
                } else {
                    if(jsonData.code == dSS11.config.codes.ERR_DOWAIT){
                        var task = new Ext.util.DelayedTask(
                            dSS11.util.checkUpdateProgress, this,
                            [ updateType, maskElement, doneCallback ]);
                        task.delay(5000);
                    } else if (jsonData.code ==
                                           dSS11.config.codes.ERR_OK) {
                        maskElement.unmask();
                        if (doneCallback !== undefined) {
                            doneCallback();
                        }
                    } else {
                        maskElement.unmask();
                        Ext.Msg.alert(_("Error"), _(jsonData.message));
                    }

                }
            } catch (error) {
                maskElement.unmask();
                Ext.Msg.alert(_("Error"),
                    _("Cold not determine current status") + ": " +
                        _(error.message));
            }
        },
        failure: function() {
             maskElement.unmask();
             Ext.Msg.alert(_("Error"), failMsg);
        }
    });
};

dSS11.util.opkg = function(action, maskElement, doneCallback, name)
{
    var message = _("Please wait");

    if ((action === 'update') || (action === 'addon-update')) {
        message = _("Downloading feed lists from server");
    } else if ((action === 'upgrade') || (action === 'addon-upgrade')) {
        message = _("Performing update, this may take several minutes");
    } else if (action === 'addon-install') {
        message = _("Please wait, installing add-on") + " <b>" + name + "</b>";
    } else if (action === 'addon-remove') {
        message = _("Please wait, removing add-on") + " <b>" + name + "</b>";
    }
    maskElement.mask(message + "...",
                     'x-mask-loading');

    Ext.Ajax.request({
        url: '/cgi-bin/opkgwrapper.sh',
        disableCaching: true,
        method: 'GET',
        scope: this,
        params: { action: action, name: name },
        success: function(response)
        {
            try {
                    var jsonData = Ext.util.JSON.decode(
                                                response.responseText);
                    if (jsonData.success !== true) {
                        Ext.Msg.alert(_("Error"), _(jsonData.message));
                        maskElement.unmask();
                        return;
                    } else {
                        var type = 'system';
                        if (action.slice(0, 5) === 'addon') {
                            type = 'addon';
                        }
                        var fn = dSS11.util.checkUpdateProgress.createDelegate(
                                            this,
                                            [
                                                type,
                                                maskElement,
                                                doneCallback
                                            ]);
                        fn();
                    }
            } catch (error) {
                maskElement.unmask();
                Ext.Msg.alert(_("Error"),
                    _("Failed to download feed lists from the server") +
                    ": " + _(error.message));
            }
        },
        failure: function() {
             maskElement.unmask();
             Ext.Msg.alert(_("Error"), _("Could not request new feed" +
                           " lists"));
        }
    });
};


Ext.namespace('dSS11', 'dSS11.config', 'dSS11.util');

dSS11.AddOnsFeedPanel = Ext.extend(Ext.Panel,
    {
        initComponent: function()
        {
            this.feedStore = new Ext.data.JsonStore(
                {
                    url: '/cgi-bin/opkgwrapper.sh?action=addon-list',
                    root: 'data',
                    fields:
                    [
                        'name', 'version', 'maintainer',
                        'description', 'homepage', 'state'
                    ]
                }
            );

            this.loaded = false;
            this.refetched = false;

            var renderAddOn = function(value, p, record)
            {
                var hp = record.get('homepage');
                if (hp !== "") {
                    hp = String.format('&nbsp;&nbsp;<a href="{0}" target="_blank">{0}</a>',
                            hp);
                }

                return String.format('<div class="{0}">' +
                        '<table border=0>' +
                        '<tr>' +
                        '<td>' +
                        '<img width="32" height="32" ' +
                             'src="/images/default_icon.png"/>' +
                        '</td>' +
                        '<td>' +
                        '<b>{1}</b>{2}<br/>' + _("Version")+':&nbsp;{3}</div>' +
                        '<br/><div class="{4}">{5}' +
                        '</td>' +
                        '</tr>' +
                        '</table>' +
                        '</div>',
                        'x-green-text x-text-link',                 // {0}
                        record.get('name'),                         // {1}
                        hp,                                         // {2}
                        record.get('version'),                      // {3}
                        'black-text',                               // {4}
                        record.get('description')                   // {5}
                );
            };

            this.mainGrid = new Ext.grid.GridPanel(
                {
                    store: this.feedStore,
                    columns:
                    [
                        {
                            id: 'addon',
                            header: _("dS add-ons"),
                            dataIndex: 'name',
                            sortable: true,
                            renderer: renderAddOn
                        }
                    ],
                    trackMouseOver: false,
                    viewConfig: { forceFit: true },
                    sm: new Ext.grid.RowSelectionModel({singleSelect:true}),
                    bodyBorder: false
                }
            );

            this.refetchTip = new Ext.ToolTip(
                {
                    target: 'refetchButton',
                    title: _("Update list of packages"),
                    html: '<br/>' +
                         _("Make sure to click this button to update the " +
                           "list of available packages."),
                    closable: true,
                    draggable: true,
                    anchor: 'top'
                }
            );

            Ext.apply(this,
                {
                    layout: 'fit',
                    tbar: [
                        {
                            xtype: 'button',
                            text: _("Refresh view"),
                            iconCls: 'icon-refresh-view',
                            scope: this,
                            handler: this.refreshList.createDelegate(this)
                        },
                        {
                            xtype: 'button',
                            text: _("Refetch list from server"),
                            id: 'refetchButton',
                            iconCls: 'icon-refresh-feed',
                            tooltip: _("Download latest package information " +
                                       "list from the server"),
                            scope: this,
                            handler: this.handleRefetch.createDelegate(this)
                        },
                        '->',
                        {
                            xtype: 'button',
                            text: _("Install selected"),
                            tooltip: _("Download and install selected add-on"),
                            iconCls: 'icon-package-add',
                            id: 'installButton',
                            scope: this,
                            handler: this.installSelected.createDelegate(this),
                            disabled: true
                        }
                    ],

                    items: [ this.mainGrid ]
                }
            );

            dSS11.AddOnsFeedPanel.superclass.initComponent.apply(this, arguments);

            this.on('activate',
                function() {
                    if (this.loaded === false) {
                        this.feedStore.load();
                        this.loaded = true;
                    }
                },
                this);

            this.feedStore.on('load',
                function()
                {
                    if ((this.feedStore.getTotalCount() === 0) &&
                        (this.refetched === false)) {
                        this.refetchTip.show();
                    }
                },
                this);

            this.mainGrid.getSelectionModel().on('selectionchange',
                function(sel)
                {
                    if (sel.hasSelection()) {
                        this.getTopToolbar().get('installButton').enable();
                    } else {
                        this.getTopToolbar().get('installButton').disable();
                    }
                },
                this);

        },
        refreshList: function()
        {
            this.feedStore.load();
        },
        refetchDone: function()
        {
            this.refetched = true;
            this.refetchTip.hide();
            this.refetchTip.disable();
            this.refreshList();
        },
        handleRefetch: function()
        {
            var fn = dSS11.util.opkg.createDelegate(this,
                           [ 'addon-update',
                              this.findParentByType('viewport').el,
                              this.refetchDone.createDelegate(this) ]);
            fn();
        },
        installSelected: function()
        {
            var sel = this.mainGrid.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.MessageBox.alert(_("No add-on selected"),
                      _("Please select an add-on that should be installed!"));
                return;
            }

            record = sel.getSelected();
            var name = record.get('name');

            var fn = dSS11.util.opkg.createDelegate(this,
                    [ 'addon-install',
                       this.findParentByType('viewport').el,
                       this.doneInstall.createDelegate(this, [ name ]),
                       name ]);
            fn();
        },
        doneInstall: function(name) {
            Ext.Msg.show({
                title: _("Info"),
                msg: _("Add-on") + " <b>" + name + "</b> " +
                     _("was successfully installed on your system"),
                icon: Ext.MessageBox.INFO,
                buttons: Ext.Msg.OK
            });
            this.refreshList();
            this.findParentByType('dss11addons').overview.refreshOverview();
        }

    }
);

Ext.reg('dss11addonsfeed', dSS11.AddOnsFeedPanel);
Ext.namespace('dSS11', 'dSS11.config');

dSS11.AddOnsUpdatePanel = Ext.extend(Ext.Panel,
    {
        initComponent: function()
        {
            this.updateStore = new Ext.data.JsonStore(
                {
                    url: '/cgi-bin/opkgwrapper.sh?action=addon-list-upgradable',
                    root: 'data',
                    fields:
                    [
                        'name', 'version', 'maintainer',
                        'description', 'homepage', 'state'
                    ]
                }
            );

            this.loaded = false;
            this.fromRefetch = false;

            var renderAddOn = function(value, p, record)
            {
                var hp = record.get('homepage');
                if (hp !== "") {
                    hp = String.format('&nbsp;&nbsp;<a href="{0}" target="_blank">{0}</a>',
                            hp);
                }

                return String.format('<div class="{0}">' +
                        '<table border=0>' +
                        '<tr>' +
                        '<td>' +
                        '<img width="32" height="32" ' +
                             'src="/images/default_icon.png"/>' +
                        '</td>' +
                        '<td>' +
                        '<b>{1}</b>{2}<br/>'+_("Version")+':&nbsp;{3}</div>' +
                        '<br/><div class="{4}">{5}' +
                        '</td>' +
                        '</tr>' +
                        '</table>' +
                        '</div>',
                        'x-green-text x-text-link',                 // {0}
                        record.get('name'),                         // {1}
                        hp,                                         // {2}
                        record.get('version'),                      // {3}
                        'black-text',                               // {4}
                        record.get('description')                   // {5}
                );
            };

            this.mainGrid = new Ext.grid.GridPanel(
                {
                    store: this.updateStore,
                    columns:
                    [
                        {
                            id: 'addon',
                            header: _("dS add-ons"),
                            dataIndex: 'name',
                            sortable: true,
                            renderer: renderAddOn
                        }
                    ],
                    trackMouseOver: false,
                    viewConfig: { forceFit: true },
                    sm: new Ext.grid.RowSelectionModel({singleSelect:true}),
                    bodyBorder: false
                }
            );

            this.noUpdatesTooltip =  new Ext.ToolTip(
                {
                    target: 'updateButton',
                    html: '<br/>' +
                          _("Your add-ons are up to date"),
                    title: _("No updates found"),
                    closable: true,
                    draggable: true,
                    anchor: 'right'
                }
            );

            this.gotUpdatesTooltip = new Ext.ToolTip(
                {
                    target: 'updateButton',
                    html: '<br/>' +
                          _("Add-on updates are available for your system, click the update button to install them"),
                    title: _("Updates found"),
                    closable: true,
                    draggable: true,
                    anchor: 'right'
                }
            );

            Ext.apply(this,
                {
                    layout: 'fit',
                    tbar: [
                        {
                            xtype: 'button',
                            text: _("Refresh view"),
                            iconCls: 'icon-refresh-view',
                            scope: this,
                            handler: this.refreshList.createDelegate(this)
                        },
                        {
                            xtype: 'button',
                            text: _("Check for new updates"),
                            iconCls: 'icon-refresh-feed',
                            scope: this,
                            tooltip: _("Contact the server and check availability of new updates"),
                            handler: this.handleRefetch.createDelegate(this)
                        },
                        '->',
                        {
                            xtype: 'button',
                            text: _("Update all"),
                            ref: 'updateButton',
                            id: 'updateButton',
                            iconCls: 'icon-package-update',
                            disabled: true,
                            scope: this,
                            handler: this.updateAddons.createDelegate(this)
                        },
                        {
                            xtype: 'button',
                            text: _("Update selected"),
                            ref: 'updateSelButton',
                            id: 'updateSelButton',
                            iconCls: 'icon-package-update',
                            disabled: true,
                            scope: this,
                            handler: this.updateSelectedAddon.createDelegate(this)
                        }
                    ],
                    items: [ this.mainGrid ]
                }
            );

            dSS11.AddOnsUpdatePanel.superclass.initComponent.apply(this, arguments);

            this.on('activate',
                function()
                {
                    if (this.loaded === false) {
                        this.updateStore.load();
                        this.loaded = true;
                    }
                },
                this);

            this.on('hide',
                function()
                {
                    this.noUpdatesTooltip.hide();
                    this.gotUpdatesTooltip.hide();
                },
                this);

            this.updateStore.on('load',
                function()
                {
                    var button = this.getTopToolbar().get('updateButton');
                    var selButton = this.getTopToolbar().get('updateSelButton');
                    var count = this.updateStore.getTotalCount();

                    if (count > 0) {
                        this.noUpdatesTooltip.hide();
                        this.noUpdatesTooltip.disable();
                        this.gotUpdatesTooltip.enable();
                        button.enable();
                        this.gotUpdatesTooltip.setTitle(
                                count + " " +
                                dSS11.config.gt.ngettext("Update",
                                                    "n% Updates", count) +
                                        " " + _("found"));

                        this.gotUpdatesTooltip.show();
                    } else {
                        button.disable();
                        selButton.disable();
                        this.gotUpdatesTooltip.hide();
                        this.gotUpdatesTooltip.disable();
                        this.noUpdatesTooltip.enable();
                        if (this.fromRefetch === true) {
                            this.noUpdatesTooltip.show();
                        }
                    }

                    if (this.fromRefetch === true) {
                        this.findParentByType('dss11addons').feeds.refreshList();
                        this.fromRefetch = false;
                    }
                },
                this);

            this.mainGrid.getSelectionModel().on('selectionchange',
                function(sel)
                {
                    if (sel.hasSelection()) {
                        this.getTopToolbar().get('updateSelButton').enable();
                    } else {
                        this.getTopToolbar().get('updateSelButton').disable();
                    }
                },
                this);
        },
        refreshList: function()
        {
            this.updateStore.load();
        },
        updateDone: function(name)
        {
            var msg = _("Add-ons successfully updated");
            if (name !== undefined) {
                msg = _("Add-on") + ' <b>' + name + '</b> ' +
                      _("successfully updated");
            }
            Ext.Msg.show({
                title: _("Info"),
                msg: msg,
                icon: Ext.MessageBox.INFO,
                buttons: Ext.Msg.OK
            });
            this.refreshList();
            this.findParentByType('dss11addons').overview.refreshOverview();
            this.findParentByType('dss11addons').feeds.refreshList();
        },
        refetchDone: function()
        {
            this.fromRefetch = true;
            this.refreshList();
        },
        handleRefetch: function()
        {
            var fn = dSS11.util.opkg.createDelegate(this,
                           [ 'addon-update',
                              this.findParentByType('viewport').el,
                              this.refetchDone.createDelegate(this) ]);
            fn();
        },
        updateAddons: function() {
            var fn = dSS11.util.opkg.createDelegate(this,
                           [ 'addon-upgrade',
                              this.findParentByType('viewport').el,
                              this.updateDone.createDelegate(this) ]);
            fn();
        },
        updateSelectedAddon: function() {
            var sel = this.mainGrid.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.MessageBox.alert(_("No add-on selected"),
                        _("Please select an add-on that should be updated!"));
                return;
            }

            record = sel.getSelected();
            var name = record.get('name');

            var fn = dSS11.util.opkg.createDelegate(this,
                           [ 'addon-upgrade',
                              this.findParentByType('viewport').el,
                              this.updateDone.createDelegate(this, [ name ]),
                              name ]);
            fn();
        }

    }
);

Ext.reg('dss11addonsupdate', dSS11.AddOnsUpdatePanel);

Ext.namespace('dSS11', 'dSS11.config');

dSS11.AddOnsPanel = Ext.extend(Ext.TabPanel,
    {
        initComponent: function()
        {
            Ext.apply(this,
                {
                    items: [
                        {
                            title: _("Overview"),
                            xtype: 'dss11addonsoverview',
                            ref: 'overview'
                        },
                        {
                            title: _("Install"),
                            xtype: 'dss11addonsfeed',
                            ref: 'feeds'
                        },
                        {
                            title: _("Update"),
                            xtype: 'dss11addonsupdate'
                        }

                    ]
                }
            );

            dSS11.AddOnsPanel.superclass.initComponent.apply(this, arguments);

            this.on('activate',
                    function() {
                        if (this.getActiveTab() === null) {
                            this.setActiveTab(this.overview);
                        }
                    },
                    this
            );
        }
    }
);

Ext.reg('dss11addons', dSS11.AddOnsPanel);

Ext.namespace('dSS11');

dSS11.NavPanel = Ext.extend(Ext.Panel,
    {
        initComponent: function()
        {

            var tabRecord = Ext.data.Record.create([
                    {
                        name: "title",
                        id: "id",
                        auto: "auto"
                    }
                ]);

            var tabStore = new Ext.data.Store({}, tabRecord);

            var cpItems =
                [
                    {
                        title: _("Overview"),
                        id: 'overview',
                        xtype: 'panel',
                        hideBorders: true,
                        ref: 'overview',
                        autoLoad: 'cgi-bin/prodinfo.hs',
                        auto: true
                    },
                    {
                        title: _("dS add-ons"),
                        id: 'addons',
                        xtype: 'panel',
                        hideBorders: true,
                        ref: 'addons',
                        xtype: 'dss11addons',
                        auto: false
                    },
                    {
                        title: _("Logged in users"),
                        id: 'loggedinusers',
                        hideBorders: true,
                        xtype: 'panel',
                        ref: 'loggedinusers',
                        autoLoad: 'cgi-bin/who.hs',
                        auto: true
                    },
                    {
                        title: _("Network settings"),
                        id: 'netconfig',
                        ref: 'netconfig',
                        hideBorders: true,
                        xtype: 'dss11netconfig',
                        auto: false
                    },
                    {
                        title: _("System settings"),
                        id: 'sysconfig',
                        ref: 'sysconfig',
                        hideBorders: true,
                        xtype: 'dss11sysconfig',
                        auto: false
                    },
                    {
                        title: _("Access settings"),
                        id: 'accessconfig',
                        ref: 'accessconfig',
                        hideBorders: true,
                        xtype: 'dss11accessconfig',
                        auto: false
                    },
                    {
                        title: _("System update"),
                        id: 'sysupdate',
                        ref: 'sysupdate',
                        hideBorders: true,
                        xtype: 'dss11systemupdate',
                        auto: false
                    }

                ];

            this.defaultSelection = 0;

            var params = Ext.urlDecode(window.location.search.substring(1));

            var param = parseInt(params.select, 10);
            if ((param >= 0) && (param < cpItems.length)) {
                this.defaultSelection = param;
            }


            var contentPanel =
            {
                ref: 'contentPanel',
                region: 'center',
                layout: 'card',
                hideBorders: true,
                autoScroll: true,
                split: true,
                items: cpItems
            };

            this.listView = new Ext.list.ListView(
                {
                    ref: 'listView',
                    store: tabStore,
                    singleSelect: true,
                    region: 'west',
                    hideHeaders: true,
                    columns: [{ header: 'name', dataIndex: 'title' }]
                }
            );

            var listHolder = new Ext.Panel(
                {
                    title: _("Menu"),
                    region: 'west',
                    layout: 'fit',
                    width: 190,
                    hideBorders: true,
                    split: true,
                    items: [ this.listView ]
                }
            );

            Ext.apply(this,
                {
                    layout: 'border',
                    items: [ listHolder, contentPanel ]
                }
            );

            dSS11.NavPanel.superclass.initComponent.apply(this, arguments);

            this.on(
                'beforerender',
                function(component)
                {
                    if (this.initializedTabs)
                    {
                        return;
                    }

                    this.contentPanel.items.each(
                        function(item)
                        {
                            var tab = new tabRecord(
                                {
                                    title: item.title,
                                    id: item.id,
                                    auto: item.auto
                                }
                            );
                            tabStore.add([tab]);
                        },
                        this
                    );

                    this.listView.on(
                        'afterrender',
                        function()
                        {
                            this.listView.select(this.defaultSelection);
                        },
                        this
                    );

                    this.listView.on(
                        'selectionchange',
                        function(listView, selections)
                        {
                            this.contentPanel.layout.setActiveItem(selections[0].viewIndex);
                        },
                        this
                    );

                    this.listView.on(
                        'click',
                        function(listView, index)
                        {
                            if (listView.getRecord(listView.getNode(index)).get('auto') === true) {
                                Ext.getCmp(listView.getRecord(listView.getNode(index)).get('id')).doAutoLoad();
                            }
                        }
                    );
                    this.initializedTabs = true;
                },
                this
            ); // this.on

        } // initcomponent
    } // extend
);

Ext.reg('dss11navpanel', dSS11.NavPanel);

Ext.namespace('dSS11', 'dSS11.config');

Ext.onReady(
    function()
    {
        Ext.get('start').remove();
        dSS11.config.dSS11TimeDiff = 0;
        dSS11.config.timeUpdateTimeout = 0;
        dSS11.config.language = 'de_DE';
        dSS11.config.langPath = '/locale/' + dSS11.config.language +
                                '/LC_MESSAGES/';

        dSS11.config.rebootWait = 60; // seconds to wait for reboot
        dSS11.config.codes = {
            "ERR_OK"        : 0,
            "ERR_MODE"      : 1,
            "ERR_MASK"      : 2,
            "ERR_IP"        : 3,
            "ERR_GW"        : 4,
            "ERR_DNS"       : 5,
            "ERR_FWRITE"    : 6,
            "ERR_NETDOWN"   : 7,
            "ERR_NETUP"     : 8,
            "ERR_PARAM"     : 9,
            "ERR_REDIRECT"  : 10,
            "ERR_RESTORE"   : 11,
            "ERR_BACKUP"    : 12,
            "ERR_HOSTNAME"  : 13,
            "ERR_APPSTART"  : 14,
            "ERR_REBOOT"    : 15,
            "ERR_HALT"      : 16,
            "ERR_NOZONES"   : 17,
            "ERR_NONTP"     : 18,
            "ERR_NONTPSRV"  : 19,
            "ERR_LINK"      : 20,
            "ERR_DATE"      : 21,
            "ERR_TIME"      : 22,
            "ERR_PASSWORD"  : 23,
            "ERR_SETUP"     : 24,
            "ERR_DOWAIT"    : 25,
            "ERR_UPDATE"    : 26
        };

        dSS11.config.opkgstate = {
            "OPKG_NOT_INSTALLED"    : 1,
            "OPKG_UNPACKED"         : 2,
            "OPKG_HALF_CONFIGURED"  : 3,
            "OPKG_INSTALLED"        : 4,
            "OPKG_HALF_INSTALLED"   : 5,
            "OPKG_CONFIG_FILES"     : 6,
            "OPKG_POST_INST_FAILED" : 7,
            "OPKG_REMOVAL_FAILED"   : 8,
            "OPKG_LAST_STATE_STATUS": 9
        };

        Ext.QuickTips.init();

        Ext.Ajax.request({
            url            : dSS11.config.langPath + 'langpack.json',
            disableCaching : true,
            scope          : this,
            success        : function(response, options) {
                try{
                    var data = Ext.decode(response.responseText);
                    var params = {
                                    "domain" : "dss11",
                                    "locale_data" : { "dss11" : data }
                                 };

                    dSS11.config.gt = new Gettext(params);

                    this.viewport = new Ext.Viewport(
                    {
                        hideBorders: true,
                        layout: 'border',
                        items:
                            [
                                {
                                    xtype: 'panel',
                                    region: 'north',
                                    height: 80,
                                    header: false,
                                    autoLoad: 'cgi-bin/mainversion.hs'
                                },
                                {
                                    region: 'center',
                                    xtype: 'dss11navpanel'
                                }
                            ]
                    });

                } catch(error) {
                    Ext.MessageBox.alert("Error", error.message);
                }
            },
            failure : function(response, options) {
                Ext.MessageBox.alert("Error", "Could not load language pack!");
            }
        });


    }
);

